enum CarouselMediaType {
  /// If the media is an image.
  image,

  /// If the media is an video.
  video
}

enum CarouselImageSource {
  /// If source of file is assets folder.
  asset,

  /// If source of file is network url.
  network,

  /// If source of file is local file path.
  file
}

class CarouselMedia {
  CarouselMedia({
    required String mediaUrl,
    required String mediaName,
    required CarouselMediaType mediaType,
    required CarouselImageSource carouselImageSource,
  }) {
    _mediaUrl = mediaUrl;
    _mediaName = mediaName;
    _carouselImageSource = carouselImageSource;
    _mediaType = mediaType;
  }

  String? _mediaUrl;
  String? _mediaName;
  CarouselImageSource? _carouselImageSource;
  CarouselMediaType? _mediaType;

  /// Network file URL or local file path.
  String? get mediaUrl => _mediaUrl;

  /// Name of the media file.
  String? get mediaName => _mediaName;

  /// File extension of the media file.
  /// Default value is [CarouselMediaType.image].
  CarouselMediaType get mediaType => _mediaType ?? CarouselMediaType.image;

  /// Image Source of the media file.
  /// Default value is [CarouselImageSource.network].
  CarouselImageSource get carouselImageSource =>
      _carouselImageSource ?? CarouselImageSource.network;

  @override
  String toString() {
    return 'CarouselMedia{_mediaUrl: $_mediaUrl, _mediaName: $_mediaName, _carouselImageSource: $_carouselImageSource, mediaType: ${mediaType.name}}';
  }
}
