import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter_carousel_media_slider/carousel_media.dart';
import 'package:video_player/video_player.dart';

class VideoPlayerHelper extends StatefulWidget {
  const VideoPlayerHelper({
    Key? key,
    required this.carouselMedia,
    required this.onComplete,
    required this.placeholder,
    required this.onError,
    required this.isLooping,
  }) : super(key: key);

  final CarouselMedia carouselMedia;
  final VoidCallback onComplete;
  final Function onError;
  final Widget placeholder;
  final bool isLooping;

  @override
  State<VideoPlayerHelper> createState() => _VideoPlayerHelperState();
}

class _VideoPlayerHelperState extends State<VideoPlayerHelper> {
  VideoPlayerController? _controller;
  VoidCallback? _listener;

  @override
  void initState() {
    super.initState();
    _initController().then((_) {
      _playController();
    });
  }

  VoidCallback _listenerSpawner() {
    return () {
      int dur = _controller!.value.duration.inMilliseconds;
      int pos = _controller!.value.position.inMilliseconds;

      if (dur - pos < 1) {
        _stopController();
        _removeController();
        widget.onComplete.call();
      }
      if (_controller!.value.hasError) {
        widget.onError(_controller!.value.errorDescription);
      }
    };
  }

  Future<void> _initController() async {
    VideoPlayerController controller;
    String fileUrl = widget.carouselMedia.mediaUrl ?? '';
    switch (widget.carouselMedia.carouselImageSource) {
      case CarouselImageSource.asset:
        controller = VideoPlayerController.asset(fileUrl);
        break;
      case CarouselImageSource.network:
        controller = VideoPlayerController.network(fileUrl);
        break;
      case CarouselImageSource.file:
        controller = VideoPlayerController.file(File(fileUrl));
        break;
    }
    try {
      await controller.initialize();
      controller.setLooping(widget.isLooping);
      _controller = controller;
      setState(() {});
    } catch (e) {
      widget.onError(e.toString());
    }
  }

  void _removeController() {
    _controller?.dispose();
    _listener = null;
  }

  void _stopController() {
    if (!widget.isLooping && _listener != null) {
      _controller?.removeListener(_listener!);
    }
    _controller?.pause();
    _controller?.seekTo(const Duration(milliseconds: 0));
  }

  void _playController() async {
    if (!widget.isLooping && _controller != null) {
      _listener = _listenerSpawner();
      _controller?.addListener(_listener!);
    }
    await _controller?.play();
  }

  @override
  Widget build(BuildContext context) {
    return Center(
      child: _controller != null && _controller!.value.isInitialized
          ? AspectRatio(
              aspectRatio: _controller!.value.aspectRatio,
              child: Center(
                child: GestureDetector(
                  onTap: () {
                    debugPrint('VideoPlayerHelper: onTap');
                    if (_controller!.value.isPlaying) {
                      _controller?.pause();
                    } else {
                      _controller?.play();
                    }
                    setState(() {});
                  },
                  child: VideoPlayer(_controller!),
                ),
              ),
            )
          : widget.placeholder,
    );
  }

  @override
  void dispose() {
    super.dispose();
    _removeController();
  }
}
