import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:youtube_player_iframe/youtube_player_iframe.dart';

class YoutubePlayerHelper extends StatefulWidget {
  const YoutubePlayerHelper({
    super.key,
    required this.url,
    required this.onComplete,
    required this.onError,
    required this.isLooping,
  });

  final String url;
  final VoidCallback onComplete;
  final Function onError;
  final bool isLooping;

  @override
  State<YoutubePlayerHelper> createState() => _YoutubePlayerHelperState();
}

class _YoutubePlayerHelperState extends State<YoutubePlayerHelper> {
  late YoutubePlayerController _controller;

  @override
  void initState() {
    super.initState();
    _controller = YoutubePlayerController(
      onWebResourceError: (error) {
        widget.onError(error);
      },
      params: YoutubePlayerParams(
        showControls: false,
        showFullscreenButton: false,
        loop: widget.isLooping,
        pointerEvents: PointerEvents.none,
        strictRelatedVideos: true,
        showVideoAnnotations: false,
        enableCaption: false,
        enableKeyboard: false,
      ),
    );
    _controller.enterFullScreen(lock: true);
    _controller.exitFullScreen(lock: true);
    trackPlayerProgress();
  }

  void trackPlayerProgress() async {
    await _controller.loadVideo(widget.url);
    if (kIsWeb) {
      _controller.playVideo();
    }
    _controller.listen(
      (event) {
        if (event.hasError) {
          debugPrint('Error: ${event.error}');
          widget.onError(event.error);
        }
        if (event.playerState == PlayerState.ended &&
            widget.isLooping == false &&
            !(event.playerState == PlayerState.buffering)) {
          Future.delayed(const Duration(milliseconds: 1500)).then((value) {
            _controller.close();
            widget.onComplete.call();
          });
        }
      },
    );
  }

  @override
  Widget build(BuildContext context) {
    return YoutubePlayer(
      controller: _controller,
      enableFullScreenOnVerticalDrag: false,
    );
  }

  @override
  void dispose() {
    super.dispose();
  }
}
